export declare type MemoFunc = [(obj: any) => boolean, (obj: any) => void];
/**
 * Helper to decycle json objects
 */
export declare function memoBuilder(): MemoFunc;
//# sourceMappingURL=memo.d.ts.map