module.exports={A:{A:{"2":"J D E F A B FC"},B:{"1":"Y Z a b c d e i j k l m n o p q r s t u v f w H","2":"C K L G M N O","1028":"W X","1540":"P Q R S T U V"},C:{"1":"gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B","2":"GC","164":"0 1 2 3 4 5 6 7 8 9 vB I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB HC IC","1540":"JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB"},D:{"1":"Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC","292":"0 1 2 3 4 5 6 7 8 9 I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB","1028":"W X","1540":"jB kB lB mB h nB oB pB qB rB P Q R S T U V"},E:{"1":"G QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC","292":"I x J D E F A B C KC 1B LC MC NC OC 2B sB","1540":"K L tB 3B","5124":"PC"},F:{"1":"pB qB rB P Q R yB S T U V W X Y Z a b c d e","2":"F B C SC TC UC VC sB DC WC tB","292":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB","1028":"nB oB","1540":"YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h"},G:{"1":"qC 4B 5B 6B 7B uB 8B 9B AC BC CC","292":"E 1B XC EC YC ZC aC bC cC dC eC fC gC hC iC","1540":"jC kC lC mC nC oC","5124":"pC"},H:{"2":"rC"},I:{"1":"H","292":"vB I sC tC uC vC EC wC xC"},J:{"292":"D A"},K:{"1":"h","2":"A B C sB DC tB"},L:{"1":"H"},M:{"1":"f"},N:{"2":"A B"},O:{"292":"yC"},P:{"1":"g 8C uB 9C AD BD","292":"I zC 0C 1C 2C 3C","1540":"2B 4C 5C 6C 7C"},Q:{"1540":"3B"},R:{"1":"CD"},S:{"1":"ED","1540":"DD"}},B:5,C:"CSS Logical Properties"};
