// prettier configuration: the same as the conventions used throughout Matrix.org
// see: https://github.com/matrix-org/eslint-plugin-matrix-org/blob/main/.prettierrc.js

module.exports = {
    printWidth: 120,
    tabWidth: 4,
    quoteProps: "consistent",
    trailingComma: "all",
};
