module.exports={A:{A:{"2":"J D E F A FC","132":"B"},B:{"1":"C K L G M N O","4":"P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H"},C:{"2":"GC vB HC","4":"0 1 2 3 4 5 6 7 8 9 J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B","8":"I x IC"},D:{"2":"I x J","4":"0 1 2 3 4 5 6 7 8 9 D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC"},E:{"2":"I x J D E F A B C K L G KC 1B LC MC NC OC 2B sB tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC"},F:{"2":"F B C SC TC UC VC sB DC WC tB","4":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e"},G:{"2":"1B XC","4":"E EC YC ZC aC bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC"},H:{"2":"rC"},I:{"2":"sC tC uC","4":"vB I H vC EC wC xC"},J:{"2":"D","4":"A"},K:{"1":"C tB","2":"A B sB DC","4":"h"},L:{"4":"H"},M:{"4":"f"},N:{"1":"B","2":"A"},O:{"4":"yC"},P:{"4":"I g zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},Q:{"4":"3B"},R:{"4":"CD"},S:{"4":"DD ED"}},B:4,C:"DeviceOrientation & DeviceMotion events"};
