module.exports={A:{A:{"2":"J D E F A B FC"},B:{"1":"P","2":"C K L G M N O Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H"},C:{"2":"0 1 2 3 4 5 6 GC vB I x J D E F A B C K L G M N O y g z xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B HC IC","66":"7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB"},D:{"1":"DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P","2":"0 1 2 I x J D E F A B C K L G M N O y g z Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC","33":"3 4 5 6 7 8 9 AB BB CB"},E:{"2":"I x J D E F A B C K L G KC 1B LC MC NC OC 2B sB tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC"},F:{"1":"0 1 2 3 4 5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB","2":"F B C hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e SC TC UC VC sB DC WC tB","33":"G M N O y g z"},G:{"2":"E 1B XC EC YC ZC aC bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC"},H:{"2":"rC"},I:{"2":"vB I H sC tC uC vC EC","33":"wC xC"},J:{"2":"D A"},K:{"2":"A B C h sB DC tB"},L:{"2":"H"},M:{"2":"f"},N:{"2":"A B"},O:{"1":"yC"},P:{"1":"zC 0C 1C 2C 3C 2B 4C 5C","2":"g 6C 7C 8C uB 9C AD BD","33":"I"},Q:{"1":"3B"},R:{"2":"CD"},S:{"1":"DD","2":"ED"}},B:7,C:"Shadow DOM (deprecated V0 spec)"};
