module.exports={A:{A:{"1":"A B","2":"J D E F FC"},B:{"1":"P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H","2":"C K L G M N O"},C:{"1":"FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B","2":"GC vB HC IC","132":"0 1 2 3 4 5 6 7 8 9 I x J D E F A B C K L G M N O y g z AB BB CB DB EB"},D:{"1":"4 5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC","2":"I","16":"0 1 2 3 x J D E z","132":"F A B C K L G M N O y g"},E:{"1":"C K L G sB tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC","2":"I x KC 1B LC","132":"J D E F A B MC NC OC 2B"},F:{"1":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e","2":"F B C SC TC UC VC sB DC WC tB"},G:{"2":"ZC aC","132":"E bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC","514":"1B XC EC YC"},H:{"2":"rC"},I:{"2":"sC tC uC","260":"vB I vC EC","514":"H wC xC"},J:{"132":"A","260":"D"},K:{"2":"A B C sB DC tB","514":"h"},L:{"260":"H"},M:{"2":"f"},N:{"514":"A","1028":"B"},O:{"2":"yC"},P:{"260":"I g zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},Q:{"260":"3B"},R:{"260":"CD"},S:{"1":"DD ED"}},B:1,C:"accept attribute for file input"};
