module.exports={A:{A:{"2":"J D E F A B FC"},B:{"1":"L G M N O","2":"C K","257":"P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H"},C:{"1":"RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B","2":"0 1 2 3 4 5 6 7 8 GC vB I x J D E F A B C K L G M N O y g z HC IC","194":"9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB"},D:{"1":"BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB","2":"0 1 2 3 4 5 6 7 8 9 I x J D E F A B C K L G M N O y g z AB","257":"XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC"},E:{"1":"D E F A B C K L G NC OC 2B sB tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC","2":"I x J KC 1B LC MC"},F:{"1":"5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB","2":"0 1 2 3 4 F B C G M N O y g z SC TC UC VC sB DC WC tB","257":"eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e"},G:{"1":"E aC bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC","2":"1B XC EC YC ZC"},H:{"2":"rC"},I:{"2":"vB I H sC tC uC vC EC wC xC"},J:{"2":"D A"},K:{"2":"A B C h sB DC tB"},L:{"1":"H"},M:{"1":"f"},N:{"2":"A B"},O:{"2":"yC"},P:{"1":"g zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD","2":"I"},Q:{"1":"3B"},R:{"2":"CD"},S:{"1":"DD ED"}},B:7,C:"Speech Synthesis API"};
