module.exports={A:{A:{"1":"B","4":"J D E F A FC"},B:{"1":"C K L G M","129":"N O P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H"},C:{"1":"0 1 2 3 4 5 6 7 8 9 J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB","4":"GC vB I x HC IC","129":"TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B"},D:{"1":"NB OB PB QB RB SB TB UB VB WB","4":"I x J","129":"0 1 2 3 4 5 6 7 8 9 D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC"},E:{"4":"I x KC 1B","129":"J D E F A B C K L G LC MC NC OC 2B sB tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC"},F:{"1":"C AB BB CB DB EB FB GB HB IB JB sB DC WC tB","4":"F B SC TC UC VC","129":"0 1 2 3 4 5 6 7 8 9 G M N O y g z KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e"},G:{"4":"1B XC EC","129":"E YC ZC aC bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC"},H:{"4":"rC"},I:{"4":"sC tC uC","129":"vB I H vC EC wC xC"},J:{"129":"D A"},K:{"1":"C sB DC tB","4":"A B","129":"h"},L:{"129":"H"},M:{"129":"f"},N:{"1":"B","4":"A"},O:{"129":"yC"},P:{"129":"I g zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},Q:{"129":"3B"},R:{"129":"CD"},S:{"1":"DD","129":"ED"}},B:1,C:"dataset & data-* attributes"};
