module.exports={A:{A:{"2":"J D E F A B FC"},B:{"2":"C K L G M N O","2052":"P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H"},C:{"2":"GC vB I x HC IC","1028":"EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B","1060":"0 1 2 3 4 5 6 7 8 9 J D E F A B C K L G M N O y g z AB BB CB DB"},D:{"2":"0 1 2 3 I x J D E F A B C K L G M N O y g z","226":"4 5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB","2052":"ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC"},E:{"2":"I x J D KC 1B LC MC","772":"K L G tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC","804":"E F A B C OC 2B sB","1316":"NC"},F:{"2":"0 1 2 3 4 5 6 7 8 9 F B C G M N O y g z AB BB CB SC TC UC VC sB DC WC tB","226":"DB EB FB GB HB IB JB KB LB","2052":"MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e"},G:{"2":"1B XC EC YC ZC aC","292":"E bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC"},H:{"2":"rC"},I:{"1":"H","2":"vB I sC tC uC vC EC wC xC"},J:{"2":"D A"},K:{"2":"A B C sB DC tB","2052":"h"},L:{"2052":"H"},M:{"1028":"f"},N:{"2":"A B"},O:{"2052":"yC"},P:{"2":"I zC 0C","2052":"g 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},Q:{"2052":"3B"},R:{"2052":"CD"},S:{"1028":"DD ED"}},B:4,C:"text-decoration styling"};
