module.exports={A:{A:{"2":"F A B FC","8":"J D E"},B:{"1":"P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H","2":"C K L G M N O"},C:{"1":"RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B","2":"GC","8":"0 1 2 3 4 5 6 7 8 9 vB I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB HC IC","194":"PB QB"},D:{"1":"EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC","8":"I x J D E F A B","257":"0 1 2 3 4 5 6 7 8 9 y g z AB BB CB DB","769":"C K L G M N O"},E:{"1":"C K L G tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC","8":"I x KC 1B LC","257":"J D E F A MC NC OC","1025":"B 2B sB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e","2":"C sB DC WC tB","8":"F B SC TC UC VC"},G:{"1":"E ZC aC bC cC dC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC","8":"1B XC EC YC","1025":"eC fC gC"},H:{"8":"rC"},I:{"1":"I H vC EC wC xC","8":"vB sC tC uC"},J:{"1":"A","8":"D"},K:{"1":"h","8":"A B C sB DC tB"},L:{"1":"H"},M:{"1":"f"},N:{"2":"A B"},O:{"1":"yC"},P:{"1":"I g zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},Q:{"1":"3B"},R:{"1":"CD"},S:{"1":"DD ED"}},B:1,C:"Details & Summary elements"};
