module.exports={A:{A:{"2":"J D E FC","260":"F A B"},B:{"132":"P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H","260":"C K L G M N O"},C:{"2":"GC vB I x HC IC","260":"0 1 2 3 4 5 6 7 8 9 J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B"},D:{"16":"I x J D E F A B C K L","132":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC"},E:{"16":"KC 1B","132":"I x J D E F A B C K L G LC MC NC OC 2B sB tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC"},F:{"1":"C WC tB","2":"F SC TC UC VC","16":"B sB DC","132":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e"},G:{"16":"1B XC","132":"E EC YC ZC aC bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC"},H:{"2":"rC"},I:{"16":"sC tC","132":"vB I H uC vC EC wC xC"},J:{"132":"D A"},K:{"1":"C tB","2":"A","16":"B sB DC","132":"h"},L:{"132":"H"},M:{"260":"f"},N:{"260":"A B"},O:{"132":"yC"},P:{"132":"I g zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},Q:{"132":"3B"},R:{"132":"CD"},S:{"260":"DD ED"}},B:5,C:"Mutation events"};
