export type ObservableFunction<T> = (val: T) => void;
export declare class SimpleObservable<T> {
    private listeners;
    constructor(initialFn?: ObservableFunction<T>);
    onUpdate(fn: ObservableFunction<T>): void;
    update(val: T): void;
    close(): void;
}
