module.exports={A:{A:{"2":"FC","8":"J D E","260":"F A B"},B:{"1":"C K L G M N O P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H"},C:{"1":"0 1 2 3 4 5 6 7 8 9 z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B","2":"GC","132":"vB HC IC","260":"I x J D E F A B C K L G M N O y g"},D:{"1":"4 5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC","132":"I x","260":"0 1 2 3 J D E F A B C K L G M N O y g z"},E:{"1":"D E F A B C K L G MC NC OC 2B sB tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC","132":"I KC 1B","260":"x J LC"},F:{"1":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e","132":"F B SC TC UC VC","260":"C sB DC WC tB"},G:{"1":"E aC bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC","132":"1B","260":"XC EC YC ZC"},H:{"132":"rC"},I:{"1":"H wC xC","132":"sC","260":"vB I tC uC vC EC"},J:{"260":"D A"},K:{"1":"h","132":"A","260":"B C sB DC tB"},L:{"1":"H"},M:{"1":"f"},N:{"260":"A B"},O:{"1":"yC"},P:{"1":"I g zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},Q:{"1":"3B"},R:{"1":"CD"},S:{"1":"DD ED"}},B:1,C:"HTML5 semantic elements"};
