import { EventStatus } from '@sentry/types';
/**
 * Converts an HTTP status code to sentry status {@link EventStatus}.
 *
 * @param code number HTTP status code
 * @returns EventStatus
 */
export declare function eventStatusFromHttpCode(code: number): EventStatus;
//# sourceMappingURL=status.d.ts.map