'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.default = pluralize;
/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

function pluralize(word, count, ending) {
  return `${count} ${word}${count === 1 ? '' : ending}`;
}
