import Bluebird from "bluebird";
import { IrcServer } from "./IrcServer";
/**
 * An IRC connection scheduler. Enables ConnectionInstance to reconnect
 * in a way that queues reconnection requests and services the FIFO queue at a
 * rate determined by ircServer.getReconnectIntervalMs().
 */
declare const _default: {
    reschedule: (a1: IrcServer, a2: number, a3: () => Promise<unknown>, a4: string) => Bluebird<unknown>;
    killAll: () => void;
};
export default _default;
