module.exports={A:{A:{"1":"B","2":"J D E F A FC"},B:{"1":"L G M N O P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H","129":"C K"},C:{"1":"SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B","2":"GC vB","260":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB","804":"I x J D E F A B C K L HC IC"},D:{"1":"YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC","260":"TB UB VB WB XB","388":"8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB","1412":"0 1 2 3 4 5 6 7 G M N O y g z","1956":"I x J D E F A B C K L"},E:{"1":"5B 6B 7B uB 8B 9B AC BC CC RC","129":"A B C K L G OC 2B sB tB 3B PC QC 4B","1412":"J D E F MC NC","1956":"I x KC 1B LC"},F:{"1":"LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e","2":"F SC TC","260":"GB HB IB JB KB","388":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB","1796":"UC VC","1828":"B C sB DC WC tB"},G:{"1":"5B 6B 7B uB 8B 9B AC BC CC","129":"dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B","1412":"E ZC aC bC cC","1956":"1B XC EC YC"},H:{"1828":"rC"},I:{"1":"H","388":"wC xC","1956":"vB I sC tC uC vC EC"},J:{"1412":"A","1924":"D"},K:{"1":"h","2":"A","1828":"B C sB DC tB"},L:{"1":"H"},M:{"1":"f"},N:{"1":"B","2":"A"},O:{"1":"yC"},P:{"1":"g 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD","260":"zC 0C","388":"I"},Q:{"1":"3B"},R:{"1":"CD"},S:{"1":"ED","260":"DD"}},B:4,C:"CSS3 Border images"};
