module.exports={A:{A:{"2":"J D E FC","132":"F A B"},B:{"1":"P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H","1028":"C K L G M N O"},C:{"1":"cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B","2":"GC vB","260":"I x J D E F A B C K L G HC IC","1028":"0 1 2 3 4 5 6 7 8 9 M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB"},D:{"1":"iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC","548":"0 1 2 3 4 5 6 I x J D E F A B C K L G M N O y g z","1028":"7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB"},E:{"1":"uB 8B 9B AC BC CC RC","2":"KC 1B","548":"I x J D E F A B C K L G LC MC NC OC 2B sB tB 3B PC QC 4B 5B 6B 7B"},F:{"1":"XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e tB","2":"F","548":"B C SC TC UC VC sB DC WC","1028":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB"},G:{"1":"uB 8B 9B AC BC CC","16":"1B","548":"E XC EC YC ZC aC bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B"},H:{"132":"rC"},I:{"1":"H","16":"sC tC","548":"vB I uC vC EC","1028":"wC xC"},J:{"548":"D A"},K:{"1":"h tB","548":"A B C sB DC"},L:{"1":"H"},M:{"1":"f"},N:{"132":"A B"},O:{"1":"yC"},P:{"1":"g 2B 4C 5C 6C 7C 8C uB 9C AD BD","1028":"I zC 0C 1C 2C 3C"},Q:{"1":"3B"},R:{"1":"CD"},S:{"1":"DD ED"}},B:4,C:"Media Queries: resolution feature"};
