import { AppServiceRegistration } from "matrix-appservice-bridge";
import { IrcBridge } from "./bridge/IrcBridge";
import { BridgeConfig } from "./config/BridgeConfig";
import { TestingOptions } from "./config/TestOpts";
export declare function generateRegistration(reg: AppServiceRegistration, config: BridgeConfig): AppServiceRegistration;
export declare function runBridge(port: number, config: BridgeConfig, reg: AppServiceRegistration, testOpts?: TestingOptions): Promise<IrcBridge>;
export declare function killBridge(ircBridge: IrcBridge, reason?: string): Promise<void>;
