module.exports={A:{A:{"2":"J D E F A B FC"},B:{"2":"C K L G M N O","1025":"d e i j k l m n o p q r s t u v f w H","1537":"P Q R S T U V W X Y Z a b c"},C:{"2":"GC","932":"0 1 2 3 4 5 6 7 8 9 vB I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB HC IC","2308":"gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B"},D:{"2":"I x J D E F A B C K L G M N O y g z","545":"0 1 2 3 4 5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB","1025":"d e i j k l m n o p q r s t u v f w H zB 0B JC","1537":"OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c"},E:{"1":"uB 8B 9B AC BC CC RC","2":"I x J KC 1B LC","516":"B C K L G sB tB 3B PC QC 4B 5B 6B 7B","548":"F A OC 2B","676":"D E MC NC"},F:{"2":"F B C SC TC UC VC sB DC WC tB","513":"CB","545":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB","1025":"e","1537":"BB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d"},G:{"1":"uB 8B 9B AC BC CC","2":"1B XC EC YC ZC","516":"oC pC qC 4B 5B 6B 7B","548":"cC dC eC fC gC hC iC jC kC lC mC nC","676":"E aC bC"},H:{"2":"rC"},I:{"2":"vB I sC tC uC vC EC","545":"wC xC","1025":"H"},J:{"2":"D","545":"A"},K:{"2":"A B C sB DC tB","1025":"h"},L:{"1025":"H"},M:{"2308":"f"},N:{"2":"A B"},O:{"1537":"yC"},P:{"545":"I","1025":"g 9C AD BD","1537":"zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB"},Q:{"1537":"3B"},R:{"1537":"CD"},S:{"932":"DD","2308":"ED"}},B:5,C:"Intrinsic & Extrinsic Sizing"};
