/**
 * Try to find a sensible representation of a message edit,
 * or returns undefined if it deems posting the entire new message
 * to be a better choice. Optimize for terseness, legibility
 * and an IRC-native feel.
 *
 * @param {string} from The original message
 * @param {string} to The new, edited version
 */
export declare function messageDiff(from: string, to: string): string | undefined;
