module.exports={A:{A:{"2":"J D E F A B FC"},B:{"1":"N O P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H","2":"C K L G M"},C:{"2":"0 1 2 3 4 5 6 7 8 9 GC vB I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB HC IC","4609":"cB dB eB fB gB hB iB jB kB","4674":"xB","5698":"bB","7490":"VB WB XB YB ZB","7746":"aB wB","8705":"lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B"},D:{"1":"hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC","2":"0 1 2 3 4 5 6 7 8 9 I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB","4097":"gB","4290":"wB bB xB","6148":"cB dB eB fB"},E:{"1":"G QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC","2":"I x J D E F A KC 1B LC MC NC OC 2B","4609":"B C sB tB","8193":"K L 3B PC"},F:{"1":"WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e","2":"0 1 2 3 4 5 6 7 8 9 F B C G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB SC TC UC VC sB DC WC tB","4097":"VB","6148":"RB SB TB UB"},G:{"1":"kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC","2":"E 1B XC EC YC ZC aC bC cC dC eC fC","4097":"gC hC iC jC"},H:{"2":"rC"},I:{"1":"H","2":"vB I sC tC uC vC EC wC xC"},J:{"2":"D A"},K:{"1":"h","2":"A B C sB DC tB"},L:{"1":"H"},M:{"4097":"f"},N:{"2":"A B"},O:{"1":"yC"},P:{"2":"I zC 0C 1C","4097":"g 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},Q:{"1":"3B"},R:{"1":"CD"},S:{"1":"ED","2":"DD"}},B:5,C:"Variable fonts"};
