import { RandomGenerator } from '../generator/RandomGenerator';
import { ArrayInt } from './internals/ArrayInt';
export declare function unsafeUniformArrayIntDistribution(from: ArrayInt, to: ArrayInt, rng: RandomGenerator): ArrayInt;
