module.exports={A:{A:{"2":"J D E F A FC","130":"B"},B:{"1":"Y Z a b c d e i j k l m n o p q r s t u v f w H","16":"C K","260":"L G","1028":"P Q R S T U V W X","5124":"M N O"},C:{"1":"o p q r s t u v f w H zB 0B","2":"0 1 2 3 4 5 6 7 8 9 GC vB I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB HC IC","5124":"m n","7172":"fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l","7746":"ZB aB wB bB xB cB dB eB"},D:{"1":"Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC","2":"0 1 2 3 4 5 6 7 8 9 I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB","260":"UB VB WB XB YB ZB aB","1028":"wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X"},E:{"2":"I x J D E F KC 1B LC MC NC OC","1028":"G PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC","3076":"A B C K L 2B sB tB 3B"},F:{"1":"pB qB rB P Q R yB S T U V W X Y Z a b c d e","2":"0 1 2 3 4 5 6 7 8 9 F B C G M N O y g z AB BB CB DB EB FB GB SC TC UC VC sB DC WC tB","260":"HB IB JB KB LB MB NB","1028":"OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB"},G:{"2":"E 1B XC EC YC ZC aC bC cC dC","16":"eC","1028":"fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC"},H:{"2":"rC"},I:{"1":"H","2":"vB I sC tC uC vC EC wC xC"},J:{"2":"D A"},K:{"1":"h","2":"A B C sB DC tB"},L:{"1":"H"},M:{"1":"f"},N:{"2":"A B"},O:{"1028":"yC"},P:{"1":"g 8C uB 9C AD BD","2":"I zC 0C","1028":"1C 2C 3C 2B 4C 5C 6C 7C"},Q:{"1028":"3B"},R:{"1":"CD"},S:{"2":"DD ED"}},B:1,C:"Streams"};
