module.exports={A:{D:{"2":"I x J D E F A B C K L G M N O y g z","33":"0 1 2 3 4 5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC"},L:{"33":"H"},B:{"2":"C K L G M N O","33":"P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H"},C:{"2":"GC","33":"0 1 2 3 4 5 6 7 8 9 vB I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B HC IC"},M:{"33":"f"},A:{"2":"J D E F A B FC"},F:{"2":"F B C SC TC UC VC sB DC WC tB","33":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e"},K:{"2":"A B C sB DC tB","33":"h"},E:{"2":"I x J KC 1B LC MC RC","33":"D E F A B C K L G NC OC 2B sB tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC"},G:{"2":"1B XC EC YC ZC","33":"E aC bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC"},P:{"2":"I","33":"g zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},I:{"2":"vB I sC tC uC vC EC","33":"H wC xC"}},B:6,C:"width: stretch property"};
