module.exports={A:{A:{"2":"J D E F FC","129":"A B"},B:{"1":"P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H","129":"C K","1025":"L G M N O"},C:{"2":"0 1 2 3 4 5 6 GC vB I x J D E F A B C K L G M N O y g z HC IC","513":"7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B"},D:{"1":"0 1 2 3 4 5 6 7 8 9 J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC","2":"I x"},E:{"1":"x J D E F A B C K L G LC MC NC OC 2B sB tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC","2":"I KC 1B"},F:{"1":"0 1 2 3 4 5 6 7 8 9 F B C G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e SC TC UC VC sB DC WC tB"},G:{"388":"E 1B XC EC YC ZC aC bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC"},H:{"2":"rC"},I:{"2":"vB sC tC uC","388":"I H vC EC wC xC"},J:{"2":"D","388":"A"},K:{"1":"A B C sB DC tB","388":"h"},L:{"388":"H"},M:{"641":"f"},N:{"388":"A B"},O:{"388":"yC"},P:{"388":"I g zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},Q:{"388":"3B"},R:{"388":"CD"},S:{"513":"DD ED"}},B:1,C:"Number input type"};
