import { MatrixAction } from "./MatrixAction";
type IrcActionType = "message" | "emote" | "topic" | "notice";
export declare class IrcAction {
    readonly type: IrcActionType;
    text: string;
    readonly ts: number;
    constructor(type: IrcActionType, text: string, ts?: number);
    static fromMatrixAction(matrixAction: MatrixAction): IrcAction | null;
}
export {};
