"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runSchema = void 0;
async function runSchema(connection) {
    // Create schema
    await connection.query(`
    CREATE TABLE room_visibility (
        room_id	TEXT UNIQUE NOT NULL,
        visibility BOOLEAN NOT NULL
    );
    `);
}
exports.runSchema = runSchema;
//# sourceMappingURL=v3.js.map