'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
Object.defineProperty(exports, 'LegacyFakeTimers', {
  enumerable: true,
  get: function () {
    return _legacyFakeTimers.default;
  }
});
Object.defineProperty(exports, 'ModernFakeTimers', {
  enumerable: true,
  get: function () {
    return _modernFakeTimers.default;
  }
});
var _legacyFakeTimers = _interopRequireDefault(require('./legacyFakeTimers'));
var _modernFakeTimers = _interopRequireDefault(require('./modernFakeTimers'));
function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}
