module.exports={A:{A:{"2":"J D E F A B FC"},B:{"2":"C K L G M N O s t u v f w H","513":"P Q R S T U V W X Y Z a b c d e i j k l m n o p q r"},C:{"1":"hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w","2":"0 1 2 3 4 5 6 7 8 9 GC vB I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB 0B HC IC","322":"PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB H zB"},D:{"2":"0 1 2 3 4 5 6 7 8 9 I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB s t u v f w H zB 0B JC","130":"GB HB IB","513":"JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j","578":"k l m n o p q r"},E:{"1":"K L G 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC","2":"I x J D E F A B C KC 1B LC MC NC OC 2B sB tB"},F:{"2":"0 1 2 3 4 5 6 7 8 9 F B C G M N O y g z AB BB CB DB EB FB GB HB JB SC TC UC VC sB DC WC tB","513":"IB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e"},G:{"1":"mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC","2":"E 1B XC EC YC ZC aC bC cC dC eC fC gC hC iC jC kC lC"},H:{"2":"rC"},I:{"2":"vB I H sC tC uC vC EC wC xC"},J:{"2":"D A"},K:{"2":"A B C h sB DC tB"},L:{"2":"H"},M:{"1":"f"},N:{"2":"A B"},O:{"2":"yC"},P:{"2":"I g zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},Q:{"2":"3B"},R:{"2":"CD"},S:{"1":"ED","322":"DD"}},B:7,C:"FIDO U2F API"};
