import { Severity } from '@sentry/types';
import { SeverityLevel } from './enums';
/**
 * Converts a string-based level into a {@link Severity}.
 *
 * @param level string representation of Severity
 * @returns Severity
 */
export declare function severityFromString(level: SeverityLevel | string): Severity;
//# sourceMappingURL=severity.d.ts.map