module.exports={A:{A:{"644":"J D E F FC","772":"A B"},B:{"1":"O P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H","260":"C K L G M N"},C:{"1":"0 1 2 3 4 5 6 7 8 9 I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B HC IC","8":"GC vB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC"},E:{"1":"I x J D E F A B C K L G KC 1B LC MC NC OC 2B sB tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC"},F:{"1":"0 1 2 3 4 5 6 7 8 9 C G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e tB","8":"F B SC TC UC VC sB DC WC"},G:{"1":"qC 4B 5B 6B 7B uB 8B 9B AC BC CC","2":"E 1B XC EC YC ZC aC bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC"},H:{"2":"rC"},I:{"2":"vB I sC tC uC vC EC wC xC","1025":"H"},J:{"2":"D A"},K:{"1":"tB","8":"A B C sB DC","1025":"h"},L:{"1025":"H"},M:{"2":"f"},N:{"1":"A B"},O:{"1025":"yC"},P:{"2":"I g zC 0C 1C 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD"},Q:{"1":"3B"},R:{"2":"CD"},S:{"2":"DD ED"}},B:1,C:"Drag and Drop"};
