module.exports={A:{A:{"2":"FC","4":"A B","8":"J D E F"},B:{"1":"M N O P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H","4":"C K L G"},C:{"4":"0 1 2 3 4 5 6 7 8 9 I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B","8":"GC vB HC IC"},D:{"1":"xB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R S T U V W X Y Z a b c d e i j k l m n o p q r s t u v f w H zB 0B JC","4":"0 1 2 3 4 5 6 7 8 9 I x J D E F A B C K L G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB wB bB"},E:{"4":"I x J D E F A B C K L G LC MC NC OC 2B sB tB 3B PC QC 4B 5B 6B 7B uB 8B 9B AC BC CC RC","8":"KC 1B"},F:{"1":"F B C UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB h nB oB pB qB rB P Q R yB S T U V W X Y Z a b c d e SC TC UC VC sB DC WC tB","4":"0 1 2 3 4 5 6 7 8 9 G M N O y g z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB"},G:{"2":"1B","4":"E XC EC YC ZC aC bC cC dC eC fC gC hC iC jC kC lC mC nC oC pC qC 4B 5B 6B 7B uB 8B 9B AC BC CC"},H:{"2":"rC"},I:{"1":"H","2":"vB I sC tC uC vC EC","4":"wC xC"},J:{"2":"D","4":"A"},K:{"1":"A B C h sB DC tB"},L:{"1":"H"},M:{"4":"f"},N:{"4":"A B"},O:{"1":"yC"},P:{"1":"g 2C 3C 2B 4C 5C 6C 7C 8C uB 9C AD BD","4":"I zC 0C 1C"},Q:{"1":"3B"},R:{"1":"CD"},S:{"4":"DD ED"}},B:1,C:"HTML5 form features"};
